from __future__ import absolute_import

import os

from celery import Celery

os.environ['DJANGO_SETTINGS_MODULE'] = 'tests.test_settings'
app = Celery('tests')

app.config_from_object('django.conf:settings')



from collections import namedtuple

from django.contrib.auth.models import User
from django.test import TestCase, override_settings
from django.urls import reverse
from model_mommy import mommy
from rest_framework import status
from rest_framework.test import APITestCase

from whisperer.models import EventQueue, Webhook, WebhookEvent
from whisperer.resources.views import (
    EventQueueViewSet,
    WebhookEventViewSet,
    WebhookViewSet,
)


class WebhookViewSetTestCase(APITestCase):
    def setUp(self):
        self.user = mommy.make(User, username="test_admin_user", is_superuser=True)
        self.dummy_user = mommy.make(User, username="dummy_user", is_superuser=False)
        dummy_user_b = mommy.make(User, username="dummy_userb", is_superuser=False)
        mommy.make(
            Webhook,
            user=self.dummy_user,
            retry_countdown_config={},
            additional_headers={},
            config={},
            _quantity=5,
        )
        mommy.make(
            Webhook,
            user=dummy_user_b,
            retry_countdown_config={},
            additional_headers={},
            config={},
            _quantity=5,
        )

    def test_list_with_admin_user(self):
        request = namedtuple("Request", ["user"])
        view = WebhookViewSet(request=request(user=self.user))
        qs = view.get_queryset()
        self.assertEqual(qs.count(), 10)

    def test_list_with_regular_user(self):
        request = namedtuple("Request", ["user"])
        view = WebhookViewSet(request=request(user=self.dummy_user))
        qs = view.get_queryset()
        self.assertEqual(qs.count(), 5)

    @override_settings(ROOT_URLCONF='whisperer.urls')
    def test_get_registry(self):
        response = self.client.get(reverse("webhook-get-registry"))
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class WebhookEventViewSetTestCase(TestCase):
    def setUp(self):
        self.user = mommy.make(User, username="test_admin_user", is_superuser=True)
        self.dummy_user = mommy.make(User, username="dummy_user", is_superuser=False)
        dummy_user_b = mommy.make(User, username="dummy_userb", is_superuser=False)
        webhook_a = mommy.make(
            Webhook,
            user=self.dummy_user,
            retry_countdown_config={},
            additional_headers={},
            config={},
        )
        webhook_b = mommy.make(
            Webhook,
            user=dummy_user_b,
            retry_countdown_config={},
            additional_headers={},
            config={},
        )
        mommy.make(WebhookEvent, webhook=webhook_a, request_payload={}, _quantity=5)
        mommy.make(WebhookEvent, webhook=webhook_b, request_payload={}, _quantity=5)

    def test_list_with_admin_user(self):
        request = namedtuple("Request", ["user"])
        view = WebhookEventViewSet(request=request(user=self.user))
        qs = view.get_queryset()
        self.assertEqual(qs.count(), 10)

    def test_list_with_regular_user(self):
        request = namedtuple("Request", ["user"])
        view = WebhookEventViewSet(request=request(user=self.dummy_user))
        qs = view.get_queryset()
        self.assertEqual(qs.count(), 5)


class EventQueueViewSetTestCase(TestCase):
    def setUp(self):
        self.user = mommy.make(User, username="test_admin_user", is_superuser=True)
        mommy.make(EventQueue, event_type="order-created")
        mommy.make(EventQueue, event_type="product-updated")

    def test_list_with_admin_user(self):
        request = namedtuple("Request", ["user"])
        view = EventQueueViewSet(request=request(user=self.user))
        qs = view.get_queryset()
        self.assertEqual(qs.count(), 2)



SECRET_KEY = 'SECRET'
BROKER_URL = 'memory://'

INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'whisperer.tests',
    'whisperer',
]

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.postgresql',
        'NAME': 'postgres',
        'USER': 'postgres',
        'PASSWORD': 'postgres',
        'HOST': 'localhost',
        'PORT': '5432',
    }
}



from django.contrib import admin

from whisperer.models import EventQueue, Webhook, WebhookEvent


@admin.register(Webhook)
class WebhookAdmin(admin.ModelAdmin):
    list_display = ("id", "user", "is_active", "event_type")


@admin.register(WebhookEvent)
class WebhookEventAdmin(admin.ModelAdmin):
    list_display = ("uuid", "webhook", "delivered", "response_http_status")


@admin.register(EventQueue)
class EventQueueAdmin(admin.ModelAdmin):
    list_display = ("uuid", "hook_id", "event_type")



from whisperer.exceptions import WebhookAlreadyRegistered
from whisperer.models import EventQueue, Webhook


class WebhookService(object):
    def register_webhook(self, user, *args, **kwargs):
        event_type = kwargs.get('event_type')
        target_url = kwargs.get('target_url')
        try:
            Webhook.objects.get(user=user, target_url=target_url, event_type=event_type)
            raise WebhookAlreadyRegistered()
        except Webhook.DoesNotExist:
            pass
        webhook = Webhook(user=user)
        for attr, value in kwargs.items():
            setattr(webhook, attr, value)
        webhook.save()
        return webhook

    def update_webhook(self, webhook, user, *args, **kwargs):
        webhook.user = user
        target_url = kwargs.get('target_url', webhook.target_url)
        event_type = kwargs.get('event_type', webhook.event_type)
        try:
            Webhook.objects.exclude(id=webhook.id).get(
                user=user, target_url=target_url, event_type=event_type
            )
            raise WebhookAlreadyRegistered()
        except Webhook.DoesNotExist:
            pass

        for attr, value in kwargs.items():
            setattr(webhook, attr, value)
        webhook.save(update_fields=kwargs.keys())
        return webhook

    def delete_webhook(self, webhook):
        webhook.is_active = False
        webhook.save(update_fields=['is_active'])


class EventQueueService(object):
    @staticmethod
    def create_event_queue(
        hook_id,
        event_type,
        app_label=None,
        model_name=None,
        instance_pk=None,
        instance_dict=None,
    ):
        if instance_dict is None:
            instance_dict = {}
        instance = EventQueue(
            hook_id=hook_id,
            event_type=event_type,
            app_label=app_label,
            model_name=model_name,
            instance_pk=instance_pk,
            instance_dict=instance_dict,
        )
        instance.save()
        return instance



from django.db.models.signals import post_save, pre_delete

from whisperer.events import WhispererEvent, registry
from whisperer.tasks import deliver_event
from whisperer.tests.models import Order
from whisperer.tests.serializers import OrderSerializer


class OrderCreateEvent(WhispererEvent):
    serializer_class = OrderSerializer
    event_type = 'order-created'
    queryset = Order.objects.select_related(
        "customer",
        "address",
    ).all()


class OrderUpdateEvent(WhispererEvent):
    serializer_class = OrderSerializer
    event_type = 'order-updated'


class OrderDeleteEvent(WhispererEvent):
    serializer_class = OrderSerializer
    event_type = 'order-deleted'


registry.register(OrderCreateEvent)
registry.register(OrderUpdateEvent)
registry.register(OrderDeleteEvent)


def signal_receiver(instance, created=False, **kwargs):
    if created:
        deliver_event(instance, 'order-created')
    else:
        deliver_event(instance, 'order-updated')


def signal_delete_receiver(sender, instance, created=False, **kwargs):
    order_data = OrderSerializer(instance).data

    deliver_event(order_data, 'order-deleted')


post_save.connect(signal_receiver, Order)
pre_delete.connect(signal_delete_receiver, sender=Order)



from django.test import TestCase
from rest_framework.exceptions import ValidationError

from whisperer.resources.serializers import WebhookSerializer


class WebhookSerializerTestCase(TestCase):
    serializer = WebhookSerializer
    data = {
        "retry_countdown_config": {"choice": "fixed", "kwargs": {"seconds": 60}},
        "target_url": "https://example.com",
    }

    def test_valid_data(self):
        self.data["event_type"] = "order-created"
        serializer = self.serializer(data=self.data)
        serializer.is_valid(raise_exception=True)

    def test_invalid_event_type(self):
        self.data["event_type"] = "invalid.event"
        serializer = self.serializer(data=self.data)
        with self.assertRaises(ValidationError):
            serializer.is_valid(raise_exception=True)
